#!/usr/bin/perl
#################################### Modules ##########################################
#
# written by markus@nikulski.net
#

require v5.10.0;

use strict;
use warnings;
use feature qw(switch);
use Net::SNMP;
use XML::Simple;
use Time::HiRes qw(gettimeofday tv_interval usleep);
use ERS_MIB 0.02;
use ERS_data 0.01;
use XML::SAX::PurePerl;

#################################### init Variables ###################################
my $logger;										# LOG object
my %cfg;										# all run parameter
my @hosts;										# load IP list for work
my @oids;										# containl all user defined OIDs
my %data;										# containl all collected information
my $version	= '0.0.2';								# version of the scrip
my $startTime;
$| = 1;

#######################################################################################
#################################### Main #############################################
#######################################################################################

initArguments();

queryDevice();

printData();

#######################################################################################
################################ Subroutines ##########################################
#######################################################################################
#initArguments ();
sub initArguments{
	$cfg{'debug'} = 0;
	my $startUp = $0;
	$startUp =~ /[^\\\/]+\.[\w]+$/;
	$startUp = $&;
	my ($syntax) = <<EOFinfo;
snmp query scrip, version $version

Usage: $startUp -h <ip/file> -c <comunity> [options]

Parameter:
  -h <ip/file>    IP or file (a listof IPs) of ERS IP address
  -c <comunity>   SNMPv2 read comunity
  -u <user>       SNMPv3 user with authprotocol and if reqired the password
  
Option:
  -m <file>       MIB file
  -p <privacy>    privicy protocol (des/3des/aes) and key, only in combination with SNMPv3
  -debug
  
Examples:
	$startUp -h 192.168.1.8 -c public
	$startUp -h ip_hosts.txt -c public -m mibQuery.xml
	$startUp -h ip_hosts.txt -c public -debug
	
	$startUp -h 192.168.1.8 -u admin1
	$startUp -h 192.168.1.8 -u admin2:md5:thesecret
	$startUp -h 192.168.1.8 -u admi3:sha:thesecret --p aes:mysecret
	$startUp -h 192.168.1.8 -u admi4:sha:thesecret --p des:mysecret
	
EOFinfo

	if(!defined($ARGV[0])){
		die $syntax;
	}else{	
		$cfg{'logLevel'} = 0;
		my $count = 0;
		foreach my $in (@ARGV){
			$cfg{'iplist'}	= $ARGV[$count + 1]	if($in =~ /^-h$/);
			$cfg{'comunity'}= $ARGV[$count + 1]	if($in =~ /^-c$/);
			$cfg{'mibFile'}	= $ARGV[$count + 1]	if($in =~ /^-m$/);
			$cfg{'debug'}	= 1			if($in =~ /^-debug$/);
			$count++;
		}
		
		print "[main] DEBUG: Script Version: $version\n" if $cfg{'debug'};
		
		if(!$cfg{'iplist'} or !$cfg{'comunity'}){
			print "ERROR: missing required parameter\n";
			die "\n$syntax\n";
		}															# for debugging only
		
		parsertr(\@hosts, $cfg{'iplist'});
		
		if(defined($cfg{'mibFile'}) and -e $cfg{'mibFile'}){
			readXmlInput($cfg{'mibFile'});
		}elsif(defined($cfg{'mibFile'})){
			die "ERROR: MIB imput file '$cfg{'mibFile'}' dosn't exist\n";
		}
		
		$startTime = gettimeofday;
	}
}

#######################################################################################
#readXmlInput()
sub readXmlInput {
	my $file = shift;
	my $xml = XML::Simple->new(noattr => 1);
	my $in = $xml->XMLin($file);
	
	print "[main] DEBUG: loading MIB queries version ".$in->{'About'}->{'Version'}." from file '$file'\n" if $cfg{'debug'};
	
	if (ref($in->{'query'}) eq 'ARRAY'){
		foreach my $item (@{$in->{'query'}}){
			push(@oids, \%{$item});
		}
	}else{
		push(@oids, \%{$in->{'query'}});
	}
}

#######################################################################################
#queryDevice ();
sub queryDevice{
	
	foreach my $ip (@hosts){
		$data{$ip} = ERS_data->new($ip,  $cfg{'debug'});
	}
	
	$cfg{'snmp'} = ERS_MIB->new($cfg{'comunity'},  $cfg{'debug'});
	if(!$cfg{'snmp'}){
		die "ERRO: by building SNMP object\n";
	}
	
	if(!$cfg{'mibFile'}){
		$cfg{'snmp'}->determineOID(\@hosts, \%data);
		
		foreach my $job (qw(getUnitCount getPortCount)){
			$cfg{'snmp'}->$job(\@hosts, \%data);
		}
	}else{
		my $count = 1;
		foreach my $query (@oids){
			my $startTimeThr = gettimeofday;
			if(defined($query->{'delay'})){
				if($query->{'delay'} =~ /^\d+$/){
					if($query->{'delay'} < 1000){
						print "[**.**.**.**] delay query $count for $query->{'delay'} ms\n";
						usleep($query->{'delay'} * 1000);
					}else{
						printf "[**.**.**.**] delay query $count for %.3f seconds\n", ($query->{'delay'} / 1000);
						print  "[**.**.**.**] next query in ";
						my $timer = $query->{'delay'} / 1000;
						for(my $count = 0; $count < $timer; $count++){
							print $timer - $count;
							usleep(1000000);
							for(my $clean = length($timer - $count); $clean > 0; $clean--){
								print "\b \b";
							}
						}
						usleep(($query->{'delay'} - ($timer * 1000)) * 1000);
						print "\r";
					}
				}else{
					print "[**.**.**.**] WARN:  delay query $count for $query->{'delay'} ms is not valid, ingnore it\n";
				}
			}
			print "[**.**.**.**] run query $count as $query->{'methode'}\n";
			if(ref($query->{'oid'}) eq 'ARRAY'){
				print "[**.**.**.**] OIDs: ";
				my $firstRun = 1;
				foreach my $oid ( @{$query->{'oid'}} ){
					if($firstRun == 1){
						print "$oid\n";
						$firstRun = 0;
					}else{
						print "\t\t    $oid\n";
					}
				}
			}else{
				print "[**.**.**.**] OID: '$query->{'oid'}'\n";
			}
			$cfg{'snmp'}->userQuery(\@hosts, \%data, $query);
			
			my $processTime = sprintf " %.3f", (gettimeofday - $startTimeThr);
			print "[**.**.**.**] query $count processed in $processTime seconds\n";
			
			$count++;
		}
	}
}

#######################################################################################
#printData
sub printData{
	my $processTime = sprintf " %.3f", (gettimeofday - $startTime);
	my $result = '';
	my $switchCount = 0;
	my $unitCount = 0;
	my $portCount = 0;
	my $energySaverEnabledCount = 0;
	my $savingsCount = 0;
	
	print "[main] processed in $processTime seconds\n";
	
	return if $cfg{'mibFile'};
	
	print "################################################################\n";
	print "###############          Collected Data          ###############\n";
	print "################################################################\n";
	
	$result = "IP\t\t units\t Ports\n";
	
	foreach my $ip (@hosts){
		next if !$data{$ip}->getValue('UnitCount');
		$switchCount++;
		$result .= "$ip \t ";
		foreach my $statement (qw(UnitCount PortCount)){
			$result .= $data{$ip}->getValue($statement)." \t ";
			given ($statement){
				when('UnitCount') { $unitCount = $unitCount + $data{$ip}->getValue($statement); }
				when('PortCount') { $portCount = $portCount + $data{$ip}->getValue($statement); }
				default {  }
			}
		}
		$result .= "\n";
	}
	
	my $sw = " $switchCount [".$cfg{'allHostCount'}."]";
	$sw .= ' ' while length($sw) < 12;
	
	$result .= "___________________________________________\n";
	$result .= "$sw\t $unitCount\t $portCount\t \n";

	print "$result\n";
}

#######################################################################################
#parsertr(<"listfile">");		extraxt a list of IP addresses 
sub parsertr{
	my $refA_hosts = shift;
	my $listfile = shift;
	my @list;
	my $problem = 0;
	
	if($listfile =~ /^(\d{1,3}\.){3}\d{1,3}$/){
		push(@$refA_hosts, $listfile);
	}else{
		open HOSTINFILE,"<$listfile"
			or $problem = 1;
		
		if($problem){
			print "ERROR: can not read host list file: '$listfile'\n";
			exit 3;
		}
		
		my @listin = <HOSTINFILE>;
		close HOSTINFILE;
		chomp @listin;
		
		my $i=0;
		foreach(@listin){
			while($listin[$i] =~ /[\n\t\s]|(#.*$)/){
				$listin[$i] =~ s/[\n\t\s]|(#.*$)//;
			}
			
			if($listin[$i] ne "" and  $listin[$i] =~ /^(\d{1,3}\.){3}\d{1,3}/){
				my $ip = $&;
				my $pwd = $';
				
				if($pwd){
					$cfg{'specificPwd', $ip} = $';
					$cfg{'specificPwd', $ip} =~ s/^;|,//;
				}
				
				my $signal = 0;
				foreach(@$refA_hosts){
					if($ip eq $_){
						$signal = 1;
						last;
					}
				}
				
				if($signal){
					print "[main] WARN:  ignore dublicate host '$ip' in file '$listfile'\n";
				}else{
					print "[main] DEBUG: add IP $ip to process list\n" if $cfg{'debug'};
					push(@$refA_hosts, $ip);
				}
			}
			$i++;
		}
	}
	
	my $length = @$refA_hosts;
	if($length){
		print "[main] DEBUG: register $length hosts\n" if $cfg{'debug'};
		$cfg{'allHostCount'} = $length;
	}else{
		print "[main] ERROR: no IP addresses found in '$listfile'\n";
		exit 5;
	}
}
